""" html_text.py formats the mail which will be sent to the end-user """
from .html_text import html
from mailthon import postman, email
from datetime import datetime


def send_mail(username, password, host, port, spark_job):
    today = datetime.today()
    date_format = '%I:%M %p &emsp; &ensp; %m/%d/%y'
    start_time = str(today.strftime(date_format))

    job_id = str(spark_job['jobId'])
    descr  = str(spark_job['jobName'])
    status = str(spark_job['status'])
    num_task = str(spark_job['numTasks'])
    num_active_tasks = str(spark_job['numActiveTasks'])
    num_comp_tasks = str(spark_job['numCompletedTasks'])
    
    msg = html.replace('!!!START_TIME!!!', start_time) \
              .replace('!!!STATUS!!!', status) \
              .replace('!!!JOB_ID!!!', job_id) \
              .replace('!!!DESCR!!!', descr) \
              .replace('!!!NUM_TASKS!!!', num_task) \
              .replace('!!!NUM_ACTIVETASKS!!!', num_active_tasks) \
              .replace('!!!NUM_COMPLETEDTASKS!!!', num_comp_tasks)
    
    p = postman(host=host, port=port, auth=(username, password))
    r = p.send(email(
        content=msg,
        subject="265, `You got Mail`",
        sender="Spark-Pager <spark@pager.com>",
        receivers=[username],
    ))
    assert r.ok
    


""" coding .:: utf-8 """
import threading
import json
import requests
from .mailer import send_mail

class Pager():
    """
    Description:
    """
    def __init__(self, spark_context, username, password, host='smtp.gmail.com', port=587):
        self.spark_context = spark_context
        self.username = username
        self.password = password
        self.host = host
        self.port = port
        self.event = True
        self.tr = threading.Thread(target=self.get_spark_jobs, name="Spark_Jobs")
            

    def parse_spark_job(self, res, completed_jobs):
        """
        Description :
        """
        current_job = json.dumps(res)
        
        if current_job in completed_jobs:
            pass
        else:
            send_mail(self.username, self.password, self.host, self.port, res)
             
                
    def get_spark_jobs(self):
        """
        Description:
        
        """
        spark_ui = self.spark_context.uiWebUrl
        app_id = self.spark_context.applicationId
        url = f'{spark_ui}/api/v1/applications/{app_id}/jobs'
        
        completed_jobs = set()
        while self.event:
            spark_jobs = self.spark_context.statusTracker().getJobIdsForGroup() ###-> [2,1,0]
            if spark_jobs:
                req = requests.get(url) \
                              .json()[0]
                
                res = {'jobId': req['jobId'], 
                       'jobName': req['name'], 
                       'status': req['status'],
                       'numTasks': req['numTasks'],
                       'numActiveTasks': req['numActiveTasks'],
                       'numCompletedTasks': req['numCompletedTasks']}
                
                self.parse_spark_job(res, completed_jobs)
                completed_jobs.add(json.dumps(res))
                

    
    def listener(self):
        """
        Description:
        
        """
        self.tr.start()
        
        
    def stop(self):
        """
        Description:
        """
        self.event = False

        


from setuptools import setup

setup(
    name='spark_pager',
    packages=['spark_pager'],
    version='1.1.1',
    description='Mail Notifier for all Spark Jobs',
    author='Bright Emah',
    author_email='bemah2020@gmail.com',
    url = 'https://github.com/BrightEmah123/spark-pager',
    python_requires='>=3',
    install_requires=['mailthon'],
    include_package_data=True
)


html = '''
<!DOCTYPE html><html lang="en">
<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, minimum-scale=1, user-scalable=no">
<head>
</head>
<body style="margin: 0;">
    <div style="font-size: 34px; text-shadow: -.03em -.03em 0 black; content: ''; left: 0; top: 0; width: 100%; height: 100%; background: radial-gradient(circle, transparent 50%, transparent 50%), radial-gradient(circle, transparent 20%, white 50%) 30px 30px; background-size: .1em .1em;border-bottom-left-radius: 50px 20px;border-bottom-right-radius: 50px 20px;">
        <div style="opacity: 27.2%;">
            <table style="width: 100%;">
                <tr>
                    <td style="font-size: 25px; font-family: Phosphate; padding-top: 20px;text-align: right;padding-right: 39px;">!!!START_TIME!!!</td>
                </tr>
            </table>
            <table style="width: 100%;">
                <tr>
                    <td style="text-align: left; font-family: Phosphate; font-size: 35px; padding-top: 42px;padding-left: 43px;">STATUS .::</td>
                    <td style="text-align: right; font-family: Phosphate; font-size: 115px;padding-right: 25px;">!!!STATUS!!!</td>
                </tr>
            </table>
            <div style="border: 14px double #999;border-radius: 23px;margin: 13px;">
                    <table style="width: 100%; font-size: 21px;border-radius: 28px;padding-left: 14px;">
                        <tr>
                            <td style="padding: 5px; font-family: Trebuchet MS;">jobID</td>
                            <td style="width: 100%;padding-left: 44px; font-family: Trebuchet MS;">!!!JOB_ID!!!</td>
                        </tr>
                        <tr>
                            <td style="padding: 5px; font-family: Trebuchet MS;">descr.</td>
                            <td style="width: 100%;padding-left: 44px; font-family: Trebuchet MS;">!!!DESCR!!!</td>
                        </tr>
                        <tr>
                            <td style="padding: 5px; font-family: Trebuchet MS;">numTasks</td>
                            <td style="width: 100%;padding-left: 44px; font-family: Trebuchet MS;">!!!NUM_TASKS!!!</td>
                        </tr>
                        <tr>
                            <td style="padding: 5px; font-family: Trebuchet MS;">numActiveTasks</td>
                            <td style="width: 100%;padding-left: 44px; font-family: Trebuchet MS;">!!!NUM_ACTIVETASKS!!!</td>
                        </tr>
                        <tr>
                            <td style="padding: 5px; font-family: Trebuchet MS;">numCompletedTasks</td>
                            <td style="width: 100%;padding-left: 44px; font-family: Trebuchet MS;">!!!NUM_COMPLETEDTASKS!!!</td>
                        </tr>
                    </table>
            </div>
            <footer style="padding: 20px; text-align:center; font-size: 17px; font-family: Trebuchet MS;">
                <i>*This email was sent from an unmonitored mailbox, please do not reply.</i>
            </footer>
        </div>
    </div>
</body>
</html>


'''